package org.projectopen.serverstatus;

/*
 * Copyright (C) 2010 ]project-open[ 
 *
 * This program is free software. You can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option)
 * any later version. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * @author Frank Bergmann (frank.bergmann@project-open.com)
 * 
 */

import java.awt.*;
import java.awt.TrayIcon.MessageType;
import java.awt.event.*;
import java.net.URL;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.table.TableColumn;

import org.projectopen.debug.DebugPanel;
import org.projectopen.debug.Logger;

@SuppressWarnings("deprecation")

/**
 * Starts the ]po[-Timesheet application as a "Tray Icon".
 * This class encapsulates user facing and GUI activities. 
 * - Setup a TrayIcon in the system tray
 * - Setup the menu for the TrayIcon
 * - Handle menu events for:
 * 		- Setup tool tips for the tray icon
 * 		- Configuration panel - Edit properties
 * 		- Time sheet log panel - log hours
 * 		- Browser login - open a browser window and perform a login
 */
public class TrayIconServiceStarter implements Logger {
    static TrayIcon trayIcon = null;
    public static JFrame debugFrame;
    public static DebugPanel debugPanel = null;
    public static JFrame servicesFrame = null;
    public static ServicesPanel servicesPanel = null;
    
    static int mouseX, mouseY;

	//Optionally set the look and feel.
    private static boolean useSystemLookAndFeel = true;

    /**
     * Provide the rest of the application with a way
     * to communicate error messages to the user.
     */
    public static void trayIconError(TrayIcon.MessageType errorType, String title, String message) {
    	TrayIcon trayIcon = TrayIconServiceStarter.getTrayIcon();
    	trayIcon.displayMessage(title, message, errorType);
    }	
	
    
    private static TrayIcon getTrayIcon() {
		return trayIcon;
	}   
    
    
    /**
     * Launch a panel that shows services status information.
     */
    private void launchServicesPanel(boolean visible) {
        if (servicesFrame == null) {
        	servicesFrame = new JFrame("Services");
        	servicesFrame.add(new ServicesPanel(this));
        	servicesFrame.pack();
            Dimension dim = servicesFrame.getSize();
            servicesFrame.setLocation(mouseX - dim.width, mouseY-dim.height);
            // servicesFrame.setPreferredSize(new Dimension(400,300));
        }
        servicesFrame.setVisible(visible);
        servicesFrame.setResizable(false);
    };
          
    /**
     * Launch a panel that shows debugging information.
     */
    private void launchDebugPanel(boolean visible) {
        if (debugFrame == null) {
        	debugFrame = new JFrame("Debug");
            debugFrame.add(debugPanel);
            debugFrame.pack();
            Dimension dim = debugFrame.getSize();
            debugFrame.setLocation(mouseX - dim.width, mouseY-dim.height);
        }
        debugFrame.setVisible(visible);    
    };
            
    public TrayIconServiceStarter() {
        if (SystemTray.isSupported()) {
        	// create a new panel and store locally for logging,
        	// even before launching the debug panel GUI.
        	debugPanel = new DebugPanel();

        	// log the application start
            this.logMessage(Logger.INFO, "Application", "Started", "");
            
            MouseListener mouseListener = new MouseListener() {
                public void mouseClicked(MouseEvent e) {
                    // System.out.println("Tray Icon - Mouse clicked!" + e.getButton());   
                	// remember the mouse position
                	mouseX = e.getX();
                	mouseY = e.getY();
                    // Take action depending on the button clicked
                    switch (e.getButton()) {
                		case 1: {
                            //Create and set up the window.
                        	launchServicesPanel(true);
                		}
                	}
                }
                public void mouseEntered(MouseEvent e) {
                    // System.out.println("Tray Icon - Mouse entered!");                 
                }
                public void mouseExited(MouseEvent e) {
                	// System.out.println("Tray Icon - Mouse exited!");                 
                }
                public void mousePressed(MouseEvent e) {
                	// System.out.println("Tray Icon - Mouse pressed!");                 
                }
                public void mouseReleased(MouseEvent e) {
                	// System.out.println("Tray Icon - Mouse released!");                 
                }

            };

            // Exit the Tray application
            ActionListener exitListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    System.out.println("Exiting...");
                    System.exit(0);
                }
            };
            
            // Show configuration frame
            ActionListener debugListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                	launchDebugPanel(true);
                }
            };
            
            // Show configuration frame
            ActionListener servicesListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                	launchServicesPanel(true);
                }
            };
            
            
            // Build the TrayIcon menu
            PopupMenu popup = new PopupMenu();
            
            MenuItem servicesItem = new MenuItem("Services");
            servicesItem.addActionListener(servicesListener);
            popup.add(servicesItem);

            MenuItem debugItem = new MenuItem("Debug");
            debugItem.addActionListener(debugListener);
            popup.add(debugItem);

            MenuItem exitItem = new MenuItem("Exit");
            exitItem.addActionListener(exitListener);
            popup.add(exitItem);

            ActionListener actionListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                	/*
                	 * Double-click on the tray icon - do nothing
                	 *
                    trayIcon.displayMessage("Action Event", 
                        "An Action Event Has Been Peformed!",
                        TrayIcon.MessageType.INFO);
                    
                    Image updatedImage = Toolkit.getDefaultToolkit().getImage("images/tray.gif");
                    trayIcon.setImage(updatedImage);
                    trayIcon.setImageAutoSize(true);
                    */
                }
            };

            // get the image stored together with the classes
            String imagePath = "/org/projectopen/timesheet/po-icon.gif";
            URL fileLocation = getClass().getResource(imagePath);
            Image image = Toolkit.getDefaultToolkit().getImage(fileLocation); 
            
            // java.net.URL imageURL = TrayIconStarter.class.getResource("images/tray.gif");          
            // Image image = Toolkit.getDefaultToolkit().getImage(imagePath);

            String imageStr = "null";
            if (image != null) { imageStr = image.getSource().toString(); }
            this.logMessage(Logger.INFO, "Application", "Loading image from "+imagePath, imageStr);
            
            
            trayIcon = new TrayIcon(image, "]project-open[ Services Panel", popup);
            trayIcon.setImageAutoSize(true);
            trayIcon.addActionListener(actionListener);
            trayIcon.addMouseListener(mouseListener);

            SystemTray tray = SystemTray.getSystemTray();
            try {
            	tray.add(trayIcon);
            } catch (AWTException e) {
                System.err.println("TrayIcon could not be added.");
            }

            // Launch a message in the Tray Icon to remind users to click here 
            TrayIcon ti = TrayIconServiceStarter.getTrayIcon();
            ti.displayMessage("]project-open[ Services Panel", "Please click here in order to check and modify the status of the ]po[ server.", TrayIcon.MessageType.INFO);
            
        } else {
            System.err.println("System tray is currently not supported.");
        }
    }
    
    /**
     * Start the time sheet logging application.
     * @param args the command line arguments
     */
    public static void main(String[] args)
    {    	
    	// Use Look and Feel?
        if (useSystemLookAndFeel) {
            try {
                UIManager.setLookAndFeel(
                    UIManager.getSystemLookAndFeelClassName());
            } catch (Exception ex) {
                System.err.println("Couldn't use system look and feel.");
            }
        }
        
        // Start the main application
        new TrayIconServiceStarter();
        
        // Do we need to cleanup after closing the application?
        // this code could go here...
    }

	/**
	 * Accepts log messages from the RESTClient and
	 * displays them to the user as a TrayIcon message
	 * if they are of level ERROR or higher.
	 */
	public void logMessage(int level, String domain, String message, String details) {

		// send message to debugging panel
		if (null != debugPanel) {
			debugPanel.logMessage(level, domain, message, details);
		}
		
		// Only show ERROR and FATAL to the user as a TrayIcon message
		if (level >= Logger.ERROR) { 
			MessageType errorType = TrayIcon.MessageType.ERROR;
			switch (level) {
			case Logger.DEBUG: errorType = TrayIcon.MessageType.INFO; 
			case Logger.INFO: errorType = TrayIcon.MessageType.INFO; 
			case Logger.WARNING: errorType = TrayIcon.MessageType.WARNING; 
			case Logger.ERROR: errorType = TrayIcon.MessageType.ERROR; 
			case Logger.FATAL: errorType = TrayIcon.MessageType.ERROR; 
			}
			TrayIcon trayIcon = TrayIconServiceStarter.getTrayIcon();
			trayIcon.displayMessage(domain, message, errorType);
		}
    }
}


